# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables

from openstack_dashboard.test.integration_tests.pages import basepage


class AuditsTable(tables.TableRegion):

    name = "audits"

    @tables.bind_table_action('launch_audit')
    def launch_audit(self, launch_button):
        launch_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)

    @tables.bind_row_action('go_to_action_plan')
    def go_to_action_plan(self, goto_button):
        goto_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)

    @tables.bind_row_action('go_to_audit_template')
    def go_to_audit_template(self, goto_button):
        goto_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)


class AuditsPage(basepage.BaseNavigationPage):

    DEFAULT_ID = "auto"
    AUDIT_TABLE_NAME_COLUMN = 'name'
    AUDIT_TABLE_TEMPLATE_COLUMN_INDEX = 1

    def __init__(self, driver, conf):
        super(AuditsPage, self).__init__(driver, conf)
        self._page_title = "Audits"

    @property
    def audits_table(self):
        return AuditsTable(self.driver, self.conf)

    def _get_audit_row(self, name):
        return self.audits_table.get_row(self.AUDIT_TABLE_NAME_COLUMN, name)

    def create_audit(self, name, id_=DEFAULT_ID, vcpus=None, ram=None,
                     root_disk=None, ephemeral_disk=None, swap_disk=None):
        create_audit_form = self.audits_table.create_audit()
        create_audit_form.name.text = name
        if id_ is not None:
            create_audit_form.audit_id.text = id_
        create_audit_form.vcpus.value = vcpus
        create_audit_form.memory_mb.value = ram
        create_audit_form.disk_gb.value = root_disk
        create_audit_form.eph_gb.value = ephemeral_disk
        create_audit_form.swap_mb.value = swap_disk
        create_audit_form.submit()
        self.wait_till_popups_disappear()

    def delete_audit(self, name):
        row = self._get_audit_row(name)
        row.mark()
        confirm_delete_audits_form = self.audits_table.delete_audit()
        confirm_delete_audits_form.submit()
        self.wait_till_popups_disappear()

    def is_audit_present(self, name):
        return bool(self._get_audit_row(name))



# Copyright 2012,  Nachi Ueno,  NTT MCL,  Inc.
# All rights reserved.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Forms for starting Watcher Audits.
"""
import logging

from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon import messages

from watcher_dashboard.api import watcher

LOG = logging.getLogger(__name__)
ADD_AUDIT_TEMPLATES_URL = "horizon:admin:audit_templates:create"


class CreateForm(forms.SelfHandlingForm):
    audit_template = forms.DynamicChoiceField(
        label=_("Audit Template"),
        add_item_link=ADD_AUDIT_TEMPLATES_URL)
    audit_name = forms.CharField(max_length=255, label=_("Name"),
                                 help_text=_("An audit name should not "
                                 "duplicate with existed audits' names."),
                                 required=False)
    audit_type = forms.ChoiceField(label=_("Audit Type"),
                                   choices=[(None, _("Select Audit Type")),
                                            ('oneshot', _('ONESHOT')),
                                            ('continuous', _('CONTINUOUS'))],
                                   widget=forms.Select(attrs={
                                       'class': 'switchable',
                                       'data-slug': 'audit_type'
                                   }))
    interval = forms.CharField(label=_("Interval (in seconds or cron format)"),
                               help_text=_("Interval in seconds or cron"
                                           "format for CONTINUOUS audit"),
                               widget=forms.TextInput(attrs={
                                   'class': 'switched',
                                   'data-switch-on': 'audit_type',
                                   'data-audit_type-continuous':
                                   _("Interval (in seconds or cron"
                                     " format)")}),
                               required=False)
    failure_url = 'horizon:admin:audits:index'
    auto_trigger = forms.BooleanField(label=_("Auto Trigger"),
                                      required=False)

    def __init__(self, request, *args, **kwargs):
        super(CreateForm, self).__init__(request, *args, **kwargs)
        audit_templates = self._get_audit_template_list(request)
        self.fields['audit_template'].choices = audit_templates

    def _get_audit_template_list(self, request):
        try:
            audit_templates = watcher.AuditTemplate.list(self.request)
        except Exception as e:
            msg = _('Failed to get audit template list: %s') % str(e)
            LOG.info(msg)
            messages.warning(request, msg)
            audit_templates = []

        choices = [
            (audit_template.uuid, audit_template.name or audit_template.uuid)
            for audit_template in audit_templates
        ]

        if choices:
            choices.insert(0, ("", _("Select Audit Template")))
        else:
            choices.insert(0, ("", _("No Audit Template found")))
        return choices

    def clean(self):
        cleaned_data = super(CreateForm, self).clean()
        audit_type = cleaned_data.get('audit_type')
        if audit_type == 'continuous' and not cleaned_data.get('interval'):
            msg = _('Please input an interval for continuous audit')
            raise forms.ValidationError(msg)
        return cleaned_data

    def handle(self, request, data):
        try:
            params = {'audit_template_uuid': data.get('audit_template')}
            params['audit_type'] = data['audit_type'].upper()
            params['auto_trigger'] = data['auto_trigger']
            params['name'] = data['audit_name']
            if data['audit_type'] == 'continuous':
                params['interval'] = data['interval']
            else:
                params['interval'] = None
            audit = watcher.Audit.create(request, **params)
            message = _('Audit was successfully created.')
            messages.success(request, message)
            return audit
        except Exception as exc:
            if getattr(exc, 'http_status', None) == 409:
                msg = _('Error: Audit name already exists.')
            else:
                msg = _('Failed to create audit.')
            LOG.info(exc)
            redirect = reverse(self.failure_url)
            exceptions.handle(request, msg, redirect=redirect)
            return False



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import copy

from watcher_dashboard.api import watcher
from watcher_dashboard.test.test_data import utils


def data(TEST):

    TEST.service_catalog.append(
        {"type": "infra-optim",
         "name": "watcher",
         "endpoints_links": [],
         "endpoints": [
             {"region": "RegionOne",
              "adminURL": "http://admin.watcher.example.com:9322",
              "internalURL": "http://int.watcher.example.com:9322",
              "publicURL": "http://public.watcher.example.com:9322"},
             {"region": "RegionTwo",
              "adminURL": "http://admin.watcher2.example.com:9322",
              "internalURL": "http://int.watcher2.example.com:9322",
              "publicURL": "http://public.watcher2.example.com:9322"}]},
    )

    TEST.goals = utils.TestDataContainer()
    TEST.efficacy_specifications = utils.TestDataContainer()
    TEST.api_goals = utils.TestDataContainer()
    efficacy_specifications_dict1 = {'name': 'spec1'}
    efficacy_specifications_dict2 = {'name': 'spec2'}
    spec1 = watcher.EfficacyIndicatorSpec(efficacy_specifications_dict1)
    spec2 = watcher.EfficacyIndicatorSpec(efficacy_specifications_dict2)
    goal_dict1 = {
        'uuid': 'gggggggg-1111-1111-1111-gggggggggggg',
        'name': 'MINIMIZE_LICENSING_COST',
        'display_name': 'Dummy',
        'efficacy_specifications': spec1
    }
    goal_dict2 = {
        'uuid': 'gggggggg-2222-2222-2222-gggggggggggg',
        'name': 'SERVER_CONSOLIDATION',
        'display_name': 'Server consolidation',
        'efficacy_specifications': spec2
    }
    TEST.api_goals.add(goal_dict1)
    TEST.api_goals.add(goal_dict2)
    _goal_dict1 = copy.deepcopy(goal_dict1)
    _goal_dict2 = copy.deepcopy(goal_dict2)

    TEST.strategies = utils.TestDataContainer()
    TEST.api_strategies = utils.TestDataContainer()
    strategy_dict1 = {
        'uuid': 'ssssssss-1111-1111-1111-ssssssssssss',
        'name': 'minimize_licensing_cost1',
        'goal_uuid': 'gggggggg-1111-1111-1111-gggggggggggg',
        'display_name': 'Fake licensing cost strategy1',
    }
    strategy_dict2 = {
        'uuid': 'ssssssss-2222-2222-2222-ssssssssssss',
        'name': 'minimize_licensing_cost2',
        'goal_uuid': 'gggggggg-1111-1111-1111-gggggggggggg',
        'display_name': 'Fake licensing cost strategy2',
    }
    strategy_dict3 = {
        'uuid': 'ssssssss-3333-3333-3333-ssssssssssss',
        'name': 'sercon',
        'goal_uuid': 'gggggggg-2222-2222-2222-gggggggggggg',
        'display_name': 'Fake Sercon',
    }
    TEST.api_strategies.add(strategy_dict1)
    TEST.api_strategies.add(strategy_dict2)
    TEST.api_strategies.add(strategy_dict3)
    _strategy_dict1 = copy.deepcopy(strategy_dict1)
    _strategy_dict2 = copy.deepcopy(strategy_dict2)
    _strategy_dict3 = copy.deepcopy(strategy_dict3)

    TEST.audit_templates = utils.TestDataContainer()
    TEST.api_audit_templates = utils.TestDataContainer()
    audit_template_dict = {
        'uuid': '11111111-1111-1111-1111-111111111111',
        'name': 'Audit Template 1',
        'description': 'Audit Template 1 description',
        'scope': '',
        'goal_uuid': 'gggggggg-1111-1111-1111-gggggggggggg',
        'strategy_uuid': 'ssssssss-1111-1111-1111-ssssssssssss',
    }
    audit_template_dict2 = {
        'uuid': '11111111-2222-2222-2222-111111111111',
        'name': 'Audit Template 2',
        'description': 'Audit Template 2 description',
        'scope': '',
        'goal_uuid': 'gggggggg-1111-1111-1111-gggggggggggg',
        'strategy_uuid': 'ssssssss-2222-2222-2222-ssssssssssss',
    }
    audit_template_dict3 = {
        'uuid': '11111111-3333-3333-3333-111111111111',
        'name': 'Audit Template 1',
        'description': 'Audit Template 3 description',
        'scope': '',
        'goal_uuid': 'gggggggg-2222-2222-2222-gggggggggggg',
        'strategy_uuid': None,
    }
    TEST.api_audit_templates.add(audit_template_dict)
    TEST.api_audit_templates.add(audit_template_dict2)
    TEST.api_audit_templates.add(audit_template_dict3)
    _audit_template_dict = copy.deepcopy(audit_template_dict)
    _audit_template_dict2 = copy.deepcopy(audit_template_dict2)
    _audit_template_dict3 = copy.deepcopy(audit_template_dict3)

    TEST.audits = utils.TestDataContainer()
    TEST.api_audits = utils.TestDataContainer()
    audit_dict = {
        'uuid': '22222222-2222-2222-2222-222222222222',
        'audit_type': 'ONESHOT',
        'name': 'Audit 1',
        'audit_template_uuid': '11111111-1111-1111-1111-111111111111',
        'interval': None,
    }
    audit_dict2 = {
        'uuid': '33333333-3333-3333-3333-333333333333',
        'audit_type': 'CONTINUOUS',
        'name': 'Audit 2',
        'audit_template_uuid': '11111111-1111-1111-1111-111111111111',
        'interval': 60,
    }
    TEST.api_audits.add(audit_dict)
    TEST.api_audits.add(audit_dict2)
    _audit_dict = copy.deepcopy(audit_dict)

    TEST.action_plans = utils.TestDataContainer()
    TEST.api_action_plans = utils.TestDataContainer()
    action_plan_dict = {
        'uuid': '33333333-3333-3333-3333-333333333333',
        'state': 'RECOMMENDED',
        'first_action_uuid': '44444444-4444-4444-4444-111111111111',
        'audit_uuid': '33333333-3333-3333-3333-333333333333'
    }
    TEST.api_action_plans.add(action_plan_dict)
    _action_plan_dict = copy.deepcopy(action_plan_dict)

    TEST.actions = utils.TestDataContainer()
    TEST.api_actions = utils.TestDataContainer()
    action_dict1 = {
        'uuid': '44444444-4444-4444-4444-111111111111',
        'state': 'PENDING',
        'next_uuid': '44444444-4444-4444-4444-222222222222',
        'action_plan_uuid': '33333333-3333-3333-3333-333333333333'
    }
    TEST.api_actions.add(action_dict1)

    action_dict2 = {
        'uuid': '44444444-4444-4444-4444-222222222222',
        'state': 'PENDING',
        'next_uuid': None,
        'action_plan_uuid': '33333333-3333-3333-3333-333333333333'
    }
    TEST.api_actions.add(action_dict2)

    action2 = watcher.Action(action_dict2)
    action1 = watcher.Action(action_dict1)
    TEST.actions.add(action1)
    TEST.actions.add(action2)

    _action_plan_dict['actions'] = [action1, action2]
    action_plan = watcher.ActionPlan(_action_plan_dict)

    _audit_dict['action_plans'] = [action_plan]
    audit = watcher.Audit(_audit_dict)

    goal1 = watcher.Goal(_goal_dict1)
    goal2 = watcher.Goal(_goal_dict2)

    strategy1 = watcher.Strategy(_strategy_dict1)
    strategy2 = watcher.Strategy(_strategy_dict2)
    strategy3 = watcher.Strategy(_strategy_dict3)

    audit_template1 = watcher.AuditTemplate(_audit_template_dict)
    audit_template2 = watcher.AuditTemplate(_audit_template_dict2)
    audit_template3 = watcher.AuditTemplate(_audit_template_dict3)

    TEST.goals.add(goal1)
    TEST.goals.add(goal2)
    TEST.strategies.add(strategy1)
    TEST.strategies.add(strategy2)
    TEST.strategies.add(strategy3)

    TEST.audit_templates.add(audit_template1)
    TEST.audit_templates.add(audit_template2)
    TEST.audit_templates.add(audit_template3)
    TEST.audits.add(audit)
    TEST.action_plans.add(watcher.ActionPlan(action_plan))



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.template.defaultfilters import title  # noqa
from django import urls
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.utils.translation import pgettext_lazy
import horizon.exceptions
import horizon.messages
import horizon.tables
from horizon.utils import filters

from watcher_dashboard.api import watcher

LOG = logging.getLogger(__name__)

ACTION_PLAN_STATE_DISPLAY_CHOICES = (
    ("NO STATE", pgettext_lazy("State of an action plan", u"No State")),
    ("ONGOING", pgettext_lazy("State of an action plan", u"On Going")),
    ("SUCCEEDED", pgettext_lazy("State of an action plan", u"Succeeded")),
    ("SUBMITTED", pgettext_lazy("State of an action plan", u"Submitted")),
    ("FAILED", pgettext_lazy("State of an action plan", u"Failed")),
    ("DELETED", pgettext_lazy("State of an action plan", u"Deleted")),
    ("RECOMMENDED", pgettext_lazy("State of an action plan", u"Recommended")),
)


class ActionPlansFilterAction(horizon.tables.FilterAction):
    # server = choices query = text
    filter_type = "server"
    filter_choices = (
        ('audit', _("Audit ="), True),
    )
    policy_rules = (("infra-optim", "action_plan:detail"),)


class ArchiveActionPlan(horizon.tables.DeleteAction):
    verbose_name = _("Archive Action Plans")
    policy_rules = (("infra-optim", "action_plan:delete"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Archive Action Plan",
            u"Archive Action Plans",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Action Plan archived",
            u"Action Plans archived",
            count
        )

    def action(self, request, obj_id):
        watcher.ActionPlan.delete(request, obj_id)


class StartActionPlan(horizon.tables.BatchAction):
    name = "start_action_plan"
    classes = ('btn-confirm',)
    policy_rules = (("infra-optim", "action_plan:update"),)
    help_text = _("Execute an action plan.")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Start Action Plan",
            u"Start Action Plans",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Action Plan started",
            u"Action Plans started",
            count
        )

    def action(self, request, action_plan_id):
        try:
            watcher.ActionPlan.start(request, action_plan_id)
        except Exception:
            msg = _('Failed to start the action plan.')
            LOG.info(msg)
            horizon.messages.warning(request, msg)

    def allowed(self, request, action_plan):
        return ((action_plan is None) or
                (action_plan.state in ("RECOMMENDED", "FAILED")))


class UpdateRow(horizon.tables.Row):
    ajax = True

    def get_data(self, request, action_plan_id):
        action_plan = None

        try:
            action_plan = watcher.Action.get(request, action_plan_id)
        except Exception:
            msg = _('Failed to get the action plan.')
            LOG.info(msg)
            horizon.messages.warning(request, msg)

        return action_plan


def format_global_efficacy(action_plan):
    formatted_global_efficacy = None
    global_efficacy = watcher.EfficacyIndicator(action_plan.global_efficacy)
    if global_efficacy.value is not None and global_efficacy.unit:
        formatted_global_efficacy = "%(value)s %(unit)s" % dict(
            unit=global_efficacy.unit,
            value=global_efficacy.value)
    elif global_efficacy.value is not None:
        formatted_global_efficacy = global_efficacy.value

    return formatted_global_efficacy


def get_audit_link(datum):
    try:
        return urls.reverse(
            "horizon:admin:audits:detail",
            kwargs={"audit_uuid": getattr(datum, "audit_uuid", None)})
    except urls.NoReverseMatch:
        return None


class ActionPlansTable(horizon.tables.DataTable):

    name = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:action_plans:detail")
    audit = horizon.tables.Column(
        'audit_uuid',
        verbose_name=_('Audit'),
        link=get_audit_link)
    updated_at = horizon.tables.Column(
        'updated_at',
        filters=(filters.parse_isotime,
                 filters.timesince_sortable),
        verbose_name=_("Updated At"))
    status = horizon.tables.Column(
        'state',
        verbose_name=_('State'),
        status=True,
        status_choices=ACTION_PLAN_STATE_DISPLAY_CHOICES)
    efficacy = horizon.tables.Column(
        transform=format_global_efficacy,
        verbose_name=_('Efficacy'))

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "action_plans"
        verbose_name = _("Action Plans")
        table_actions = (
            # CancelActionPlan,
            ActionPlansFilterAction,
            StartActionPlan,
            ArchiveActionPlan,
        )
        row_actions = (
            StartActionPlan,
            # CreateActionPlans,
            ArchiveActionPlan,
            # CreateActionPlans,
            # DeleteActionPlans,
        )
        row_class = UpdateRow


class RelatedActionPlansTable(horizon.tables.DataTable):

    name = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:action_plans:detail")
    audit = horizon.tables.Column(
        'audit_uuid',
        verbose_name=_('Audit'),
        link=get_audit_link)
    updated_at = horizon.tables.Column(
        'updated_at',
        filters=(filters.parse_isotime,
                 filters.timesince_sortable),
        verbose_name=_("Updated At"))
    status = horizon.tables.Column(
        'state',
        verbose_name=_('State'),
        status=True,
        status_choices=ACTION_PLAN_STATE_DISPLAY_CHOICES)
    efficacy = horizon.tables.Column(
        transform=format_global_efficacy,
        verbose_name=_('Efficacy'))

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "related_action_plans"
        verbose_name = _("Related Action Plans")
        hidden_title = False
        row_actions = (
            StartActionPlan,
            ArchiveActionPlan,
        )
        row_class = UpdateRow


class RelatedEfficacyIndicatorsTable(horizon.tables.DataTable):

    name = horizon.tables.Column(
        'name',
        verbose_name=_("Name"))

    description = horizon.tables.Column(
        'description',
        verbose_name=_("Description"))

    unit = horizon.tables.Column(
        'unit',
        verbose_name=_("Unit"))

    value = horizon.tables.Column(
        'value',
        verbose_name=_("Value"))

    def get_object_id(self, datum):
        return datum.name

    class Meta(object):
        name = "related_efficacy_indicators"
        verbose_name = _("Related Efficacy Indicators")
        hidden_title = False



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
import horizon.exceptions
import horizon.messages
import horizon.tables

from watcher_dashboard.api import watcher


class CreateAuditTemplates(horizon.tables.LinkAction):
    name = "create"
    verbose_name = _("Create Template")
    url = "horizon:admin:audit_templates:create"
    classes = ("ajax-modal", "btn-launch")
    policy_rules = (("infra-optim", "audit_template:create"),)


class AuditTemplatesFilterAction(horizon.tables.FilterAction):
    filter_type = "server"
    filter_choices = (
        ('goal', _("Goal ="), True),
        ('strategy', _("Strategy ="), True),
    )
    policy_rules = (("infra-optim", "audit_template:detail"),)


class LaunchAudit(horizon.tables.BatchAction):
    name = "launch_audit"
    verbose_name = _("Launch Audit")
    data_type_singular = _("Launch Audit")
    data_type_plural = _("Launch Audits")
    success_url = "horizon:admin:audits:index"
    policy_rules = (("infra-optim", "audit:create"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Launch Audit",
            "Launch Audits",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Launched Audit",
            "Launched Audits",
            count
        )

    def action(self, request, obj_id):
        params = {'audit_template_uuid': obj_id}
        params['audit_type'] = 'ONESHOT'
        params['auto_trigger'] = False
        watcher.Audit.create(request, **params)


class ArchiveAuditTemplates(horizon.tables.DeleteAction):
    verbose_name = _("Archive Templates")
    policy_rules = (("infra-optim", "audit_template:delete"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Archive Template",
            "Archive Templates",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Archived Template",
            "Archived Templates",
            count
        )

    def delete(self, request, obj_id):
        watcher.AuditTemplate.delete(request, obj_id)


class AuditTemplatesTable(horizon.tables.DataTable):
    name = horizon.tables.Column(
        'name',
        verbose_name=_("Name"),
        link="horizon:admin:audit_templates:detail")
    goal = horizon.tables.Column(
        'goal_name',
        verbose_name=_('Goal'),
        status=True,
    )
    strategy = horizon.tables.Column(
        'strategy_name',
        verbose_name=_('Strategy'),
        status=True,
    )

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "audit_templates"
        verbose_name = _("Audit Templates")
        table_actions = (
            CreateAuditTemplates,
            AuditTemplatesFilterAction,
            LaunchAudit,
            ArchiveAuditTemplates,
        )
        row_actions = (
            LaunchAudit,
            ArchiveAuditTemplates,
        )



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from django import urls

from watcher_dashboard import api
from watcher_dashboard.test import helpers as test

INDEX_URL = urls.reverse('horizon:admin:goals:index')
DETAILS_VIEW = 'horizon:admin:goals:detail'


class GoalsTest(test.BaseAdminViewTests):

    @mock.patch.object(api.watcher.Goal, 'list')
    def test_index(self, mock_list):
        mock_list.return_value = self.goals.list()

        res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'infra_optim/goals/index.html')
        goals = res.context['goals_table'].data
        self.assertCountEqual(goals, self.goals.list())

    @mock.patch.object(api.watcher.Goal, 'list')
    def test_goal_list_unavailable(self, mock_list):
        mock_list.side_effect = self.exceptions.watcher

        resp = self.client.get(INDEX_URL)
        self.assertMessageCount(resp, error=1, warning=0)

    @mock.patch.object(api.watcher.Strategy, 'list')
    @mock.patch.object(api.watcher.Goal, 'get')
    def test_details(self, mock_get, mock_list):
        goal = self.goals.first()
        goal_id = goal.uuid
        mock_get.return_value = goal

        DETAILS_URL = urls.reverse(DETAILS_VIEW, args=[goal_id])
        res = self.client.get(DETAILS_URL)
        self.assertTemplateUsed(res, 'infra_optim/goals/details.html')
        goals = res.context['goal']
        self.assertCountEqual([goals], [goal])

    @mock.patch.object(api.watcher.Goal, 'get')
    def test_details_exception(self, mock_get):
        at = self.goals.first()
        at_id = at.uuid
        mock_get.side_effect = self.exceptions.watcher

        DETAILS_URL = urls.reverse(DETAILS_VIEW, args=[at_id])
        res = self.client.get(DETAILS_URL)
        self.assertRedirectsNoFollow(res, INDEX_URL)



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import django.forms
from horizon import tables
from horizon.tables import formset as hformset

from watcher_dashboard.test import helpers as test


class FormsetTableTests(test.TestCase):

    def test_populate(self):
        """Create a FormsetDataTable and populate it with data."""

        class TableObj(object):
            pass

        obj = TableObj()
        obj.name = 'test object'
        obj.value = 42
        obj.id = 4

        class TableForm(django.forms.Form):
            name = django.forms.CharField()
            value = django.forms.IntegerField()

        TableFormset = django.forms.formsets.formset_factory(TableForm,
                                                             extra=0)

        class Table(hformset.FormsetDataTable):
            formset_class = TableFormset

            name = tables.Column('name')
            value = tables.Column('value')

            class Meta(object):
                name = 'table'

        table = Table(self.request)
        table.data = [obj]
        formset = table.get_formset()
        self.assertEqual(len(formset), 1)
        form = formset[0]
        form_data = form.initial
        self.assertEqual(form_data['name'], 'test object')
        self.assertEqual(form_data['value'], 42)



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from horizon.test import helpers as test
from selenium.common import exceptions as selenium_exceptions


class BrowserTests(test.SeleniumTestCase):
    def test_jasmine(self):
        url = "%s%s" % (self.live_server_url, "/jasmine/")
        self.selenium.get(url)
        wait = self.ui.WebDriverWait(self.selenium, 10)

        def jasmine_done(driver):
            text = driver.find_element_by_id("jasmine-testresult").text
            return "Tests completed" in text

        wait.until(jasmine_done)

        failed_elem = self.selenium.find_element_by_class_name("failed")
        failed = int(failed_elem.text)
        if failed:
            self.log_failure_messages()
        self.assertEqual(failed, 0)

    def log_failure_messages(self):
        logger = logging.getLogger('selenium')
        logger.error("Errors found during jasmine test:")
        fail_elems = self.selenium.find_elements_by_class_name("fail")
        for elem in fail_elems:
            try:
                module = elem.find_element_by_class_name("module-name").text
            except selenium_exceptions.NoSuchElementException:
                continue
            message = elem.find_element_by_class_name("test-message").text
            source = elem.find_element_by_tag_name("pre").text
            logger.error("Module: %s, message: %s, source: %s" % (
                module, message, source))



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path

from watcher_dashboard.content.strategies import views


urlpatterns = [
    re_path(r'^$',
            views.IndexView.as_view(), name='index'),
    re_path(r'^(?P<strategy_uuid>[^/]+)/detail$',
            views.DetailView.as_view(), name='detail'),
]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from selenium.webdriver.common import by

from openstack_dashboard.test.integration_tests.pages import basepage
from openstack_dashboard.test.integration_tests.regions import forms
from openstack_dashboard.test.integration_tests.regions import tables


class AuditTemplatesTable(tables.TableRegion):

    name = 'audit_templates'

    CREATE_AUDIT_TEMPLATE_FORM_FIELDS = ("name", "description",
                                         "goal_id", "strategy_id")

    @tables.bind_table_action('create')
    def create_audit_template(self, create_button):
        create_button.click()
        return forms.FormRegion(
            self.driver, self.conf,
            field_mappings=self.CREATE_AUDIT_TEMPLATE_FORM_FIELDS)

    @tables.bind_table_action('delete')
    def delete_audit_template(self, delete_button):
        delete_button.click()
        return forms.BaseFormRegion(self.driver, self.conf, None)

    @tables.bind_row_action('launch_audit')
    def launch_audit(self, launch_button, row):
        launch_button.click()
        return forms.BaseFormRegion(self.driver, self.conf)


class AudittemplatesPage(basepage.BaseNavigationPage):

    DEFAULT_DESCRIPTION = "Fake description from integration tests"
    DEFAULT_GOAL = "SERVER_CONSOLIDATION"

    AUDITS_PAGE_TITLE = "Audits - OpenStack Dashboard"

    AUDIT_TEMPLATE_INFO_SUB_TITLE = "Audit Template Info"

    # Set fields name attribute
    CREATE_AUDIT_TEMPLATE_FORM_FIELDS = (
        "name", "description", "goal"
    )

    _audittemplates_info_title_locator = (by.By.CSS_SELECTOR, 'div.detail>h4')

    def __init__(self, driver, conf):
        super(AudittemplatesPage, self).__init__(driver, conf)
        self._page_title = "Audit Templates"

    @property
    def audittemplates_table(self):
        return AuditTemplatesTable(self.driver, self.conf)

    @property
    def audit_templates__action_create_form(self):
        return forms.FormRegion(self.driver, self.conf, None,
                                self.CREATE_AUDIT_TEMPLATE_FORM_FIELDS)

    def _get_row_with_audit_template_name(self, name):
        self._turn_off_implicit_wait()
        row = self.audittemplates_table.get_row("name", name)
        self._turn_on_implicit_wait()
        return row

    def delete_audit_template(self, name):
        row = self._get_row_with_audit_template_name(name)
        row.mark()
        confirm_delete_audit_template_form = (
            self.audittemplates_table.delete_audit_template())
        confirm_delete_audit_template_form.submit()

    def create_audit_template(self,
                              name,
                              description=DEFAULT_DESCRIPTION,
                              goal_id=DEFAULT_GOAL):
        self.audittemplates_table.create_audit_template()
        self.audit_templates__action_create_form.name.text = name
        self.audit_templates__action_create_form.description.text = description
        self.audit_templates__action_create_form.goal_id.value = goal_id
        self.audit_templates__action_create_form.submit()

    def is_audit_template_present(self, name):
        return bool(
            self._get_row_with_audit_template_name(name))

    def launch_audit(self, name):
        row = self._get_row_with_audit_template_name(name)
        self.audittemplates_table.launch_audit(row)
        # Check that the name appears in Audits page
        return (self.driver.title == self.AUDITS_PAGE_TITLE) \
            and (name in self.driver.page_source)

    def show_audit_template_info(self, name):
        self.driver.find_element_by_link_text(name).click()
        info_line = self._get_element(*self._audittemplates_info_title_locator)
        return self._is_text_visible(
            info_line, self.AUDIT_TEMPLATE_INFO_SUB_TITLE)



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.utils.translation import gettext_lazy as _
import horizon.exceptions
from horizon import forms
import horizon.tables
import horizon.tabs
from horizon.utils import memoized
import horizon.workflows

from watcher_dashboard.api import watcher
from watcher_dashboard.content.action_plans import tables
from watcher_dashboard.content.actions import tables as action_tables
from watcher_dashboard.content.audits import forms as wforms

LOG = logging.getLogger(__name__)


class IndexView(horizon.tables.DataTableView):
    table_class = tables.ActionPlansTable
    template_name = 'infra_optim/action_plans/index.html'
    page_title = _("Action Plans")

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)
        context['action_plans_count'] = self.get_action_plans_count()
        return context

    def get_data(self):
        action_plans = []
        search_opts = self.get_filters()
        try:
            action_plans = watcher.ActionPlan.list(
                self.request, **search_opts)
        except Exception as exc:
            LOG.exception(exc)
            horizon.exceptions.handle(
                self.request,
                _("Unable to retrieve action_plan information."))
        return action_plans

    def get_action_plans_count(self):
        return len(self.get_data())

    def get_filters(self):
        filters = {}
        filter_action = self.table._meta._filter_action
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field):
                filter_string = self.table.get_filter_string()
                if filter_field and filter_string:
                    filters[filter_field] = filter_string
        return filters


class ArchiveView(forms.ModalFormView):
    form_class = wforms.CreateForm
    form_id = "create_audit_form"
    modal_header = _("Create Audit")
    template_name = 'infra_optim/audits/create.html'
    page_title = _("Create Audit")
    submit_label = _("Create Audit")


class DetailView(horizon.tables.MultiTableView):
    table_classes = (
        action_tables.RelatedActionsTable,
        tables.RelatedEfficacyIndicatorsTable)
    template_name = 'infra_optim/action_plans/details.html'
    page_title = _("Action Plan Details: {{ action_plan.uuid }}")

    @memoized.memoized_method
    def _get_data(self):
        action_plan_uuid = None
        try:
            action_plan_uuid = self.kwargs['action_plan_uuid']
            action_plan = watcher.ActionPlan.get(
                self.request, action_plan_uuid)
        except Exception as exc:
            LOG.exception(exc)
            msg = _('Unable to retrieve details for action_plan "%s".') \
                % action_plan_uuid
            horizon.exceptions.handle(
                self.request, msg,
                redirect=self.redirect_url)
        return action_plan

    def get_related_wactions_data(self):
        try:
            action_plan = self._get_data()
            actions = watcher.Action.list(self.request,
                                          action_plan=action_plan.uuid)
        except Exception as exc:
            LOG.exception(exc)
            actions = []
            msg = _('Action list can not be retrieved.')
            horizon.exceptions.handle(self.request, msg)
        return actions

    def get_related_efficacy_indicators_data(self):
        try:
            action_plan = self._get_data()
            efficacy_indicators = [
                watcher.EfficacyIndicator(indicator)
                for indicator in action_plan.efficacy_indicators]
        except Exception as exc:
            LOG.exception(exc)
            msg = _('Failed to get the efficacy indicators: %s') % str(exc)
            LOG.info(msg)
            horizon.messages.warning(self.request, msg)

        return efficacy_indicators

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        action_plan = self._get_data()
        context["action_plan"] = action_plan
        LOG.info('*********************************')
        LOG.info(action_plan)
        LOG.info('*********************************')
        return context

    def get_tabs(self, request, *args, **kwargs):
        action_plan = self._get_data()
        return self.tab_group_class(
            request, action_plan=action_plan, **kwargs)



# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from openstack_dashboard.test import helpers

from watcher_dashboard import api
from watcher_dashboard.test.test_data import utils


class WatcherTestsMixin(object):
    def _setup_test_data(self):
        super(WatcherTestsMixin, self)._setup_test_data()
        utils.load_test_data(self)


class TestCase(WatcherTestsMixin, helpers.TestCase):
    pass


class APITestCase(WatcherTestsMixin, helpers.APITestCase):
    def setUp(self):
        super(APITestCase, self).setUp()

        self._original_watcherclient = api.watcher.watcherclient

        api.watcher.watcherclient = lambda request: self.stub_watcherclient()

    def tearDown(self):
        super(APITestCase, self).tearDown()
        api.watcher.watcherclient = self._original_watcherclient

    def stub_watcherclient(self):
        if not hasattr(self, "watcherclient"):
            self.watcherclient = mock.Mock()
        return self.watcherclient


class BaseAdminViewTests(WatcherTestsMixin, helpers.BaseAdminViewTests):
    pass



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

# The slug of the panel to be added to HORIZON_CONFIG. Required.
PANEL = 'audit_templates'
# The slug of the dashboard the PANEL associated with. Required.
PANEL_DASHBOARD = 'admin'
# The slug of the panel group the PANEL is associated with.
PANEL_GROUP = 'watcher'

# Python panel class of the PANEL to be added.
ADD_PANEL = 'watcher_dashboard.content.audit_templates.panel.AuditTemplates'



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template.defaultfilters import title  # noqa
from django.utils.translation import gettext_lazy as _
import horizon.exceptions
import horizon.messages
import horizon.tables


class GoalsTable(horizon.tables.DataTable):
    uuid = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:goals:detail")

    name = horizon.tables.Column(
        'name',
        verbose_name=_('Name'))

    display_name = horizon.tables.Column(
        'display_name',
        verbose_name=_('Verbose Name'))

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "goals"
        verbose_name = _("Goals")


class EfficacySpecificationTable(horizon.tables.DataTable):

    name = horizon.tables.Column(
        'name',
        verbose_name=_("Name"))

    description = horizon.tables.Column(
        'description',
        verbose_name=_("Description"))

    unit = horizon.tables.Column(
        'unit',
        verbose_name=_("Unit"))

    schema = horizon.tables.Column(
        'schema',
        verbose_name=_("Schema"))

    def get_object_id(self, datum):
        return datum.name

    class Meta(object):
        name = "efficacy_specification"
        verbose_name = _("Efficacy specification")
        hidden_title = False



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from openstack_dashboard.test.test_data import keystone_data
from openstack_dashboard.test.test_data import utils


def load_test_data(load_onto=None):
    from watcher_dashboard.test.test_data import exceptions
    from watcher_dashboard.test.test_data import watcher_data

    # The order of these loaders matters, some depend on others.
    loaders = (exceptions.data,
               keystone_data.data,
               watcher_data.data)
    if load_onto:
        for data_func in loaders:
            data_func(load_onto)
        return load_onto
    else:
        return utils.TestData(*loaders)


class TestDataContainer(utils.TestDataContainer):
    def filter(self, filtered=None, **kwargs):
        """Returns objects in this container """
        """whose attributes match the given keyword arguments.
        """
        if filtered is None:
            filtered = self._objects
        try:
            key, value = kwargs.popitem()
        except KeyError:
            # We're out of filters, return
            return filtered

        def get_match(obj):
            return key in obj and obj.get(key) == value

        return self.filter(filtered=filter(get_match, filtered), **kwargs)

    def delete(self):
        """Delete the first object from this container and return a list"""
        self._objects.remove(self._objects[0])
        return self._objects



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django import shortcuts
from django.template.defaultfilters import title  # noqa
from django import urls
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.utils.translation import pgettext_lazy
import horizon.exceptions
import horizon.messages
import horizon.tables
from horizon.utils import filters

from watcher_dashboard.api import watcher

AUDIT_STATE_DISPLAY_CHOICES = (
    ("NO STATE", pgettext_lazy("State of an audit", u"No State")),
    ("ONGOING", pgettext_lazy("State of an audit", u"On Going")),
    ("SUCCEEDED", pgettext_lazy("State of an audit", u"Succeeeded")),
    ("SUBMITTED", pgettext_lazy("State of an audit", u"Submitted")),
    ("FAILED", pgettext_lazy("State of an audit", u"Failed")),
    ("DELETED", pgettext_lazy("State of an audit", u"Deleted")),
    ("PENDING", pgettext_lazy("State of an audit", u"Pending")),
)


class AuditsFilterAction(horizon.tables.FilterAction):
    # server = choices query = text
    filter_type = "server"
    filter_choices = (
        ('audit_template', _("Audit Template ="), True),
    )
    policy_rules = (("infra-optim", "audit:detail"),)


class CreateAudit(horizon.tables.LinkAction):
    name = "create_audit"
    verbose_name = _("Create Audit")
    url = "horizon:admin:audits:create"
    classes = ("ajax-modal", "btn-launch")
    policy_rules = (("infra-optim", "audit:create"),)


class GoToActionPlan(horizon.tables.Action):
    name = "go_to_action_plan"
    verbose_name = _("Go to Action Plan")
    url = "horizon:admin:action_plans:detail"
    policy_rules = (("infra-optim", "action_plan:detail"),)

    def allowed(self, request, audit):
        return audit or audit.state in ("SUCCEEDED", )

    def single(self, table, request, audit_id):
        try:
            action_plans = watcher.ActionPlan.list(
                request,
                audit=audit_id)
        except Exception:
            horizon.exceptions.handle(
                request,
                _("Unable to retrieve action_plan information."))
            return "javascript:void(0);"

        return shortcuts.redirect(urls.reverse(
            self.url,
            args=[action_plans[0].uuid]))


class ArchiveAudits(horizon.tables.DeleteAction):
    verbose_name = _("Archive Audits")
    policy_rules = (("infra-optim", "audit:delete"),)

    def allowed(self, request, audit):
        return audit or audit.state not in ("ONGOING", "PENDING")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            "Archive Audit",
            "Archive Audits",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            "Archived Audit",
            "Archived Audits",
            count
        )

    def delete(self, request, obj_id):
        watcher.Audit.delete(request, obj_id)


class AuditsTable(horizon.tables.DataTable):
    uuid = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:audits:detail")
    name = horizon.tables.Column(
        'name',
        verbose_name=_("Name"),
        link="horizon:admin:audits:detail")
    goal = horizon.tables.Column(
        'goal_name',
        verbose_name=_('Goal'))
    strategy = horizon.tables.Column(
        'strategy_name',
        verbose_name=_('Strategy'))
    audit_type = horizon.tables.Column(
        'audit_type',
        verbose_name=_('Audit Type'))
    auto_trigger = horizon.tables.Column(
        'auto_trigger',
        verbose_name=_('Auto Trigger'))
    status = horizon.tables.Column(
        'state',
        verbose_name=_('State'),
        status=True,
        status_choices=AUDIT_STATE_DISPLAY_CHOICES)

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "audits"
        verbose_name = _("Audits")
        launch_actions = (CreateAudit,)
        table_actions = launch_actions + (
            AuditsFilterAction,
        )
        row_actions = (
            GoToActionPlan,
            ArchiveAudits,
        )


class RelatedAuditsTable(horizon.tables.DataTable):
    name = horizon.tables.Column(
        'uuid',
        verbose_name=_("UUID"),
        link="horizon:admin:audits:detail")
    audit_template = horizon.tables.Column(
        'audit_template_name',
        verbose_name=_('Audit Template'),
        filters=(title, filters.replace_underscores))
    status = horizon.tables.Column(
        'state',
        verbose_name=_('State'),
        status=True,
        status_choices=AUDIT_STATE_DISPLAY_CHOICES)

    def get_object_id(self, datum):
        return datum.uuid

    class Meta(object):
        name = "audits"
        verbose_name = _("Related audits")
        hidden_title = False



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from watcher_dashboard import api
from watcher_dashboard.test import helpers as test


class WatcherAPITests(test.APITestCase):

    def test_goal_list(self):
        goals = {'goals': self.api_goals.list()}
        watcherclient = self.stub_watcherclient()
        watcherclient.goal.list = mock.Mock(
            return_value=goals)

        ret_val = api.watcher.Goal.list(self.request)
        self.assertIsInstance(ret_val, dict)
        self.assertIn('goals', ret_val)
        for n in ret_val['goals']:
            self.assertIsInstance(n, dict)
        watcherclient.goal.list.assert_called_with(
            detail=True)

    def test_goal_get(self):
        goal = self.api_goals.first()
        goal_id = self.api_goals.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.goal.get = mock.Mock(
            return_value=goal)

        ret_val = api.watcher.Goal.get(self.request, goal_id)
        self.assertIsInstance(ret_val, dict)
        watcherclient.goal.get.assert_called_with(
            goal_id)

    def test_strategy_list(self):
        strategies = {'strategies': self.api_strategies.list()}
        watcherclient = self.stub_watcherclient()

        watcherclient.strategy.list = mock.Mock(
            return_value=strategies)

        ret_val = api.watcher.Strategy.list(self.request)
        self.assertIn('strategies', ret_val)
        for n in ret_val['strategies']:
            self.assertIsInstance(n, dict)
        watcherclient.strategy.list.assert_called_with(
            detail=True)

    def test_strategy_get(self):
        strategy = self.api_strategies.first()
        strategy_id = self.api_strategies.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.strategy.get = mock.Mock(
            return_value=strategy)

        ret_val = api.watcher.Strategy.get(self.request, strategy_id)
        self.assertIsInstance(ret_val, dict)
        watcherclient.strategy.get.assert_called_with(
            strategy_id)

    def test_audit_template_list(self):
        audit_templates = {
            'audit_templates': self.api_audit_templates.list()}
        watcherclient = self.stub_watcherclient()

        watcherclient.audit_template.list = mock.Mock(
            return_value=audit_templates)

        ret_val = api.watcher.AuditTemplate.list(self.request)

        self.assertIn('audit_templates', ret_val)
        for n in ret_val['audit_templates']:
            self.assertIsInstance(n, dict)
        watcherclient.audit_template.list.assert_called_with(
            detail=True)

    def test_audit_template_list_with_filters(self):
        search_opts = {'name': 'Audit Template 1'}
        audit_templates = {
            'audit_templates': self.api_audit_templates.filter(**search_opts)}
        watcherclient = self.stub_watcherclient()

        watcherclient.audit_template.list = mock.Mock(
            return_value=audit_templates)

        ret_val = api.watcher.AuditTemplate.list(
            self.request, **search_opts)

        self.assertIn('audit_templates', ret_val)
        for n in ret_val['audit_templates']:
            self.assertIsInstance(n, dict)

        self.assertEqual(ret_val, audit_templates)
        watcherclient.audit_template.list.assert_called_with(
            detail=True, **search_opts)

    def test_audit_template_get(self):
        audit_template = self.api_audit_templates.first()
        audit_template_id = self.api_audit_templates.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.audit_template.get = mock.Mock(
            return_value=audit_template)

        ret_val = api.watcher.AuditTemplate.get(self.request,
                                                audit_template_id)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit_template.get.assert_called_with(
            audit_template_id=audit_template_id)

    def test_audit_template_create(self):
        audit_template = self.api_audit_templates.first()
        name = audit_template['name']
        goal = audit_template['goal_uuid']
        strategy = audit_template['strategy_uuid']
        description = audit_template['description']
        scope = audit_template['scope']

        watcherclient = self.stub_watcherclient()
        watcherclient.audit_template.create = mock.Mock(
            return_value=audit_template)

        ret_val = api.watcher.AuditTemplate.create(
            self.request, name, goal, strategy,
            description, scope)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit_template.create.assert_called_with(
            name=name,
            goal=goal,
            strategy=strategy,
            description=description,
            scope=scope)

    def test_audit_template_patch(self):
        audit_template = self.api_audit_templates.first()
        audit_template_id = self.api_audit_templates.first()['uuid']
        form_data = {'name': 'new Audit Template 1'}

        watcherclient = self.stub_watcherclient()
        watcherclient.audit_template.patch = mock.Mock(
            return_value=audit_template)

        ret_val = api.watcher.AuditTemplate.patch(
            self.request, audit_template_id,
            form_data)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit_template.patch.assert_called_with(
            audit_template_id,
            [{'name': 'name', 'value': 'new Audit Template 1'}]
        )

    def test_audit_template_delete(self):
        audit_template_list = self.api_audit_templates.list()
        audit_template_id = self.api_audit_templates.first()['uuid']
        deleted_at_list = self.api_audit_templates.delete()

        watcherclient = self.stub_watcherclient()
        watcherclient.audit_template.delete = mock.Mock()
        api.watcher.AuditTemplate.delete(self.request,
                                         audit_template_id)
        self.assertEqual(audit_template_list, deleted_at_list)
        self.assertEqual(len(audit_template_list), len(deleted_at_list))
        watcherclient.audit_template.delete.assert_called_with(
            audit_template_id=audit_template_id)

    def test_audit_list(self):
        audits = {'audits': self.api_audits.list()}

        watcherclient = self.stub_watcherclient()

        watcherclient.audit.list = mock.Mock(
            return_value=audits)

        ret_val = api.watcher.Audit.list(self.request)

        self.assertIn('audits', ret_val)
        for n in ret_val['audits']:
            self.assertIsInstance(n, dict)
        watcherclient.audit.list.assert_called_with(
            detail=True)

    def test_audit_get(self):
        audit = self.api_audits.first()
        audit_id = self.api_audits.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.audit.get = mock.Mock(
            return_value=audit)

        ret_val = api.watcher.Audit.get(self.request, audit_id)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit.get.assert_called_with(
            audit=audit_id)

    def test_audit_create(self):
        audit = self.api_audits.first()
        audit_template_id = self.api_audit_templates.first()['uuid']

        audit_type = self.api_audits.first()['audit_type']
        audit_name = self.api_audits.first()['name']
        audit_template_uuid = audit_template_id

        watcherclient = self.stub_watcherclient()
        watcherclient.audit.create = mock.Mock(
            return_value=audit)

        ret_val = api.watcher.Audit.create(
            self.request, audit_template_uuid, audit_type, audit_name)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit.create.assert_called_with(
            audit_template_uuid=audit_template_uuid,
            audit_type=audit_type, auto_trigger=False, name=audit_name)

    def test_audit_create_with_interval(self):
        audit = self.api_audits.list()[1]
        audit_template_id = self.api_audit_templates.first()['uuid']

        audit_type = self.api_audits.first()['audit_type']
        audit_name = self.api_audits.first()['name']
        interval = audit['interval']
        audit_template_uuid = audit_template_id

        watcherclient = self.stub_watcherclient()
        watcherclient.audit.create = mock.Mock(
            return_value=audit)

        ret_val = api.watcher.Audit.create(
            self.request, audit_template_uuid, audit_type, audit_name,
            False, interval)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit.create.assert_called_with(
            audit_template_uuid=audit_template_uuid,
            audit_type=audit_type,
            auto_trigger=False,
            interval=interval,
            name=audit_name)

    def test_audit_create_with_auto_trigger(self):
        audit = self.api_audits.list()[1]
        audit_template_id = self.api_audit_templates.first()['uuid']

        audit_type = self.api_audits.first()['audit_type']
        audit_name = self.api_audits.first()['name']
        audit_template_uuid = audit_template_id

        watcherclient = self.stub_watcherclient()
        watcherclient.audit.create = mock.Mock(
            return_value=audit)

        ret_val = api.watcher.Audit.create(
            self.request, audit_template_uuid, audit_type, audit_name, True)
        self.assertIsInstance(ret_val, dict)
        watcherclient.audit.create.assert_called_with(
            audit_template_uuid=audit_template_uuid,
            audit_type=audit_type,
            auto_trigger=True,
            name=audit_name)

    def test_audit_delete(self):
        audit_id = self.api_audits.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.audit.delete = mock.Mock()

        api.watcher.Audit.delete(self.request, audit_id)
        watcherclient.audit.delete.assert_called_with(
            audit=audit_id)

    def test_action_plan_list(self):
        action_plans = {'action_plans': self.api_action_plans.list()}

        watcherclient = self.stub_watcherclient()

        watcherclient.action_plan.list = mock.Mock(
            return_value=action_plans)

        ret_val = api.watcher.ActionPlan.list(self.request)

        self.assertIn('action_plans', ret_val)
        for n in ret_val['action_plans']:
            self.assertIsInstance(n, dict)
        watcherclient.action_plan.list.assert_called_with(
            detail=True)

    def test_action_plan_get(self):
        action_plan = self.api_action_plans.first()
        action_plan_id = self.api_action_plans.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.action_plan.get = mock.Mock(
            return_value=action_plan)

        ret_val = api.watcher.ActionPlan.get(self.request, action_plan_id)
        self.assertIsInstance(ret_val, dict)
        watcherclient.action_plan.get.assert_called_with(
            action_plan_id=action_plan_id)

    def test_action_plan_start(self):
        action_plan_id = self.api_action_plans.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.action_plan.start = mock.Mock()

        api.watcher.ActionPlan.start(self.request, action_plan_id)
        watcherclient.action_plan.start.assert_called_with(
            action_plan_id)

    def test_action_plan_delete(self):
        action_plan_id = self.api_action_plans.first()['uuid']

        watcherclient = self.stub_watcherclient()
        watcherclient.action_plan.delete = mock.Mock()

        api.watcher.ActionPlan.delete(self.request, action_plan_id)
        watcherclient.action_plan.delete.assert_called_with(
            action_plan_id=action_plan_id)

    def test_action_list(self):
        actions = {'actions': self.api_actions.list()}
        watcherclient = self.stub_watcherclient()

        watcherclient.action.list = mock.Mock(
            return_value=actions)

        ret_val = api.watcher.Action.list(self.request)

        self.assertIn('actions', ret_val)
        for n in ret_val['actions']:
            self.assertIsInstance(n, dict)
        watcherclient.action.list.assert_called_with(
            detail=True)
